package mmtext.images.descriptor;


import java.awt.Point;

import mmtext.images.IBinaryImage;
import mmtext.images.IImage;

/**
/**
 * Implementacao do paper Morphology-Based Text Line Extraction
 * 
 * @author Wonder Alexandre Luz Alves
 * @advisor Ronaldo Fumio Hashimoto
 *  
 * @description
 * Classe responsavel por implementar os momentos da imagem
 *
 */
public class Moment{
   
    /**
     * Calculando uma estimacao da orientacao do componente conexo (em radiano)
     * Esse procedimento e baseado em momentos estatiscos
     * @param img - imagem de entrada
     * @return orientacao
     */
    public static double getOrientationEstimationInRadian(IBinaryImage img){
        double xx = 0, yy = 0;
        double u20 = 0, u02 = 0, u11 = 0;
        double thetaR = 0;
         
        //calculando o momento central
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                xx += x * img.getPixelBinary(x, y);
                yy += y * img.getPixelBinary(x, y); 
            }
        }
        double m00 = img.getArea();
        xx = xx / m00;
        yy = yy / m00;
        
        u20 = getMoment(img, xx, yy, 2, 0);
        u02 = getMoment(img, xx, yy, 0, 2);
        u11 = getMoment(img, xx, yy, 1, 1);
       
        //calculando a orientacao
        thetaR = 0.5 * Math.atan2( 2.0 * u11, u20 - u02 );
        return thetaR;
    }
    
    public static Point getCentroid(IBinaryImage img){
    	double xx = 0, yy = 0;
         
        //calculando o momento central
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                xx += x * img.getPixelBinary(x, y);
                yy += y * img.getPixelBinary(x, y); 
            }
        }
        double m00 = img.getArea();
        xx = xx / m00;
        yy = yy / m00;
        return new Point((int)xx, (int)yy);
    }
    
    /**
     * Retorna a excentricidade do componente conexa. Razao entre o eixo maior com o eixo menor
     * @param img
     * @return
     */
    public static double eccentricity(IBinaryImage img){
    	double xx = 0, yy = 0;
       double u20 = 0, u02 = 0, u11 = 0;
         
        //calculando o momento central
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                xx += x * img.getPixelBinary(x, y);
                yy += y * img.getPixelBinary(x, y); 
            }
        }
        double m00 = img.getArea();
        xx = xx / m00;
        yy = yy / m00;
        
        //Calculado os momentos de segunda ordem 
        u20 = getMoment(img, xx, yy, 2, 0);
        u02 = getMoment(img, xx, yy, 0, 2);
        u11 = getMoment(img, xx, yy, 1, 1);
       
        return (Math.pow(u02 - u20, 2) + (4 * u11)) / img.getArea();
    }
    
    /**
     * Calculando uma estimacao da orientacao do componente conexo (em graus)
     * Esse procedimento e baseado em momentos estatiscos
     * @param img - imagem de entrada
     * @return orientacao
     */
    public static double getOrientationForRegionText(IBinaryImage img){
        double thetaR = getOrientationEstimationInRadian(img);
        double thetaG = thetaR * 180.0 / Math.PI; //convertendo de radianos para graus
        
        if(thetaR > 0)
            thetaG = 360 - thetaG; 
        else
            thetaG *= -1;
        return thetaG;  
    }
    
    public static double toDegreesForRegionText(double inRadian){
        double thetaG = inRadian * 180.0 / Math.PI;
        if(inRadian > 0)
            thetaG = 360 - thetaG; 
        else
            thetaG *= -1;
        return thetaG; 
    }
    
    /**
     * Pega o momento referidos ao centroide
     * @param img
     * @param xCentroide
     * @param yCentroide
     * @param p
     * @param q
     * @return
     */
    public static double getMoment(IBinaryImage img, double xCentroide, double yCentroide, int p, int q){
        double result = 0;
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                result += Math.pow(x - xCentroide, p) * Math.pow(y - yCentroide, q) * img.getPixelBinary(x, y);
            }
        }
        return result;
    }
    
    
    public static Point getPointRotation(double graus, IImage img, int w, int h){
        double radian = 2.0 * Math.PI * graus / 360.0;
        double ct = Math.cos(radian);
        double st = Math.sin(radian);
        int hc = img.getHeight()/2;
        int wc = img.getWidth()/2;
        
        int nh = (int) ((h - hc) * ct - (w - wc) * st + hc);
        int nw = (int) ((h - hc) * st + (w - wc) * ct + wc);
        
        return new Point(nw, nh);
            
        
    }

}

 